import Vue from "vue";
import Vuex from "vuex";
import axios from "axios";
import createPersist from "vuex-localstorage";
import {
  KEYBOARDS_LOCALSTORAGE_NAMESPACE,
  DEFAULT_KEYBOARDS,
  NEW_KEYBOARD,
  UTF8_ASSETS_URL,
} from "../config";

Vue.use(Vuex);

const initialState = {
  keyboards: DEFAULT_KEYBOARDS,
  // Select the keyboard at index 0 in the keyboard list by default
  selectedKeyboard: 0,
  scriptsList: null,
  scripts: {},
  // { name: scriptName, page: 1 }
  selectedScript: null,
};

export default new Vuex.Store({
  plugins: [
    createPersist({
      namespace: KEYBOARDS_LOCALSTORAGE_NAMESPACE,
      initialState,
      // Never expires
      // https://github.com/crossjs/vuex-localstorage/blob/master/src/index.js#L29
      expires: 0,
    }),
  ],
  state: initialState,
  mutations: {
    setScriptsList(state, data) {
      state.scriptsList = data;
    },
    setScript(state, { scriptName, data }) {
      // Override scripts objects to trigger reactivity
      state.scripts = {
        ...state.scripts,
        [scriptName]: data,
      };
    },
    setSelectedScript(state, { name, page }) {
      state.selectedScript = { name, page };
    },
    setSelectedKeyboard(state, index) {
      state.selectedKeyboard = index;
    },
    addKeyboard(state, keyboardToAdd = null) {
      if (!keyboardToAdd) keyboardToAdd = NEW_KEYBOARD;
      // Copy the list as we want to avoid using the same reference
      state.keyboards.push({
        ...keyboardToAdd,
        characters: [...keyboardToAdd.characters],
      });
    },
    removeKeyboard(state, index) {
      state.keyboards.splice(index, 1);
      // Reset selected keyboard to default if the user deleted the selected one
      if (state.selectedKeyboard === index) state.selectedKeyboard = 0;
    },
    updateKeyboardName(state, { index, name }) {
      if (!state.keyboards[index]) return;
      state.keyboards[index].name = name;
    },
    updateKeyboardKey(state, { index, ...newKey }) {
      // Copy the keyboard if the index is correct to avoid modifying it directly in the state
      const keyboard = state.keyboards[index] && { ...state.keyboards[index] };
      if (!keyboard) return;
      // Find the key and replace it
      const keyPos = keyboard.characters.findIndex(
        (k) => k.row === newKey.row && k.column === newKey.column
      );
      if (keyPos >= 0) keyboard.characters[keyPos] = newKey;
      else keyboard.characters.push(newKey);
      state.keyboards[index] = keyboard;
      // Update list reference to propagate changes across components
      state.keyboards = [...state.keyboards];
    },
    removeKeyboardKey(state, { index, ...keyToRemove }) {
      // Copy the keyboard if the index is correct to avoid modifying it directly in the state
      const keyboard = state.keyboards[index] && { ...state.keyboards[index] };
      if (!keyboard) return;
      // Find the key and remove it
      const keyPos = keyboard.characters.findIndex(
        (k) => k.row === keyToRemove.row && k.column === keyToRemove.column
      );
      if (keyPos < 0) return;
      keyboard.characters.splice(keyPos, 1);
      state.keyboards[index] = keyboard;
      // Update list reference to propagate changes across components
      state.keyboards = [...state.keyboards];
    },
  },
  actions: {
    async listScripts({ state, commit }) {
      // List the names of all available UTF-8 glyphs
      if (state.scriptsList !== null) return;
      const resp = await axios.get(`${UTF8_ASSETS_URL}/index.json`);
      commit("setScriptsList", resp.data);
    },
    async retrieveScript({ state, commit }, scriptName) {
      /*
       * Retrieve a complete set of glyphs for a specific script
       * Note that scripts are compressed using gzip
       */
      if (state.scripts[scriptName]) return;
      const resp = await axios.get(`${UTF8_ASSETS_URL}/${scriptName}.json`);
      commit("setScript", { scriptName, data: resp.data });
    },
  },
  modules: {},
  getters: {},
});
